package com.yizhi.album.application.controller;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.album.application.domain.AlbumClassify;
import com.yizhi.album.application.service.IAlbumClassifyService;
import com.yizhi.album.application.vo.AlbumClassifyVo;
import com.yizhi.album.application.vo.ApiAlbumClassifyVo;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.util.application.beanutil.BeanCopyListUtil;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 专辑分类 前端控制器
 * </p>
 *
 * @author hc123
 * @since 2019-12-30
 */
@RestController
@RequestMapping("/albumClassify")
public class AlbumClassifyController {

	private Logger logger = LoggerFactory.getLogger(AlbumClassifyController.class);

	@Autowired
	IdGenerator idGenerator;

	@Autowired
	private IAlbumClassifyService albumClassifyService;

	@GetMapping("/find/id")
	public com.yizhi.album.application.vo.domain.AlbumClassify getById(@RequestParam(name = "id") Long id) {
		com.yizhi.album.application.vo.domain.AlbumClassify albumClassify = new com.yizhi.album.application.vo.domain.AlbumClassify();
		BeanUtils.copyProperties(albumClassifyService.selectById(id), albumClassify);
		return albumClassify;
	}

	@ApiOperation("添加分类")
	@RequestMapping(value = "/save", method = RequestMethod.POST)
	public int saveClassify(@RequestBody AlbumClassify classify) {
		return albumClassifyService.saveClassify(classify);
	}

	/**
	 * 检查是否存在分类名称
	 * 
	 * @param name       分类名称
	 * @param parentId   分类的上级id（一级分类为0）
	 * @param siteId     站点id
	 * @param type       检查类型（1：新增，2：修改(需要传classifyId)）
	 * @param classifyId 分类id，修改检查时用到
	 * @return 不存在返回true，存放相同名返回false
	 */
	@GetMapping("/check/name")
	public boolean checkExtisClassifyName(@RequestParam(name = "name") String name,
			@RequestParam(name = "parentId", required = false) Long parentId, @RequestParam(name = "siteId") Long siteId,
			@RequestParam(name = "type") Integer type, @RequestParam(name = "classifyId", required = false) Long classifyId) {
		return albumClassifyService.checkExtisClassifyName(name, parentId, siteId, type, classifyId);
	}

	@ApiOperation("分类删除")
	@RequestMapping(value = "/delete", method = RequestMethod.POST)
	public int deleteClassify(@RequestBody AlbumClassify classify) {
		return albumClassifyService.deleteClassify(classify.getId());
	}

	@ApiOperation("分类名称修改")
	@RequestMapping(value = "/update", method = RequestMethod.POST)
	public int updateClassify(@RequestBody AlbumClassify classify) {
		return albumClassifyService.updateClassify(classify);
	}

	@ApiOperation("分类查询")
	@RequestMapping(value = "/list", method = RequestMethod.GET)
	public Page<com.yizhi.album.application.vo.domain.AlbumClassify> listClassify(@RequestParam(required = false, name = "isStartHQ") Boolean isStartHQ,
																				  @RequestParam(required = false,name = "name") String name, @RequestParam(name = "pageNo") Integer pageNo, @RequestParam(name = "pageSize") Integer pageSize,
																				  @RequestParam(name = "companyId") Long companyId, @RequestParam(name = "siteId") Long siteId,
																				  @RequestParam(name = "orgIds", required = false) List<Long> orgIds) {
		Page<com.yizhi.album.application.vo.domain.AlbumClassify> page = new Page<>();
		Page<AlbumClassify> albumClassifyPage = albumClassifyService.listClassify(name, pageNo, pageSize, companyId, orgIds, siteId, isStartHQ);
		BeanUtils.copyProperties(albumClassifyPage, page);
		page.setRecords(BeanCopyListUtil.copyListProperties(albumClassifyPage.getRecords(), com.yizhi.album.application.vo.domain.AlbumClassify::new));

		return page;
	}

	@ApiOperation("分类排序")
	@RequestMapping(value = "/sort", method = RequestMethod.POST)
	public int sortClassify(@RequestBody List<AlbumClassify> list) {
		logger.info("管理端传入要排序的ID列表==========================:" + list);
		return albumClassifyService.updateClassiyfsort(list);
	}

	@ApiOperation("分类分页查询")
	@RequestMapping(value = "/page/list", method = RequestMethod.GET)
	public Page<ApiAlbumClassifyVo> pageList(@RequestParam(name = "name", required = false) String name,
			@RequestParam(name = "pageNo") Integer pageNo, @RequestParam(name = "pageSize") Integer pageSize,
			@RequestParam(name = "companyId") Long companyId, @RequestParam(name = "siteId") Long siteId,
			@RequestParam(name = "orgIds", required = false) List<Long> orgIds) {
		logger.info("分类分页查询orgIds=", orgIds == null ? 0 : orgIds.size());
		Page<ApiAlbumClassifyVo> page = albumClassifyService.selectListByCompanyId(companyId, name, orgIds, siteId,
				pageNo, pageSize);

		List<AlbumClassify> classifies2 = albumClassifyService.selectClassifyByLayer(companyId, siteId, 2);

		List<AlbumClassify> classifies3 = albumClassifyService.selectClassifyByLayer(companyId, siteId, 3);
		Map<Long, List<ApiAlbumClassifyVo>> map3 = new HashMap<Long, List<ApiAlbumClassifyVo>>();
		for (AlbumClassify c3 : classifies3) {
			if (map3.containsKey(c3.getParentId())) {
				List<ApiAlbumClassifyVo> apiClassifyVos = map3.get(c3.getParentId());
				ApiAlbumClassifyVo apiClassifyVo = new ApiAlbumClassifyVo();
				apiClassifyVo = copyBean(apiClassifyVo, c3);
				apiClassifyVos.add(apiClassifyVo);
			} else {
				List<ApiAlbumClassifyVo> apiClassifyVos = new ArrayList<ApiAlbumClassifyVo>();
				ApiAlbumClassifyVo apiClassifyVo = new ApiAlbumClassifyVo();
				apiClassifyVo = copyBean(apiClassifyVo, c3);
				apiClassifyVos.add(apiClassifyVo);
				map3.put(c3.getParentId(), apiClassifyVos);
			}
		}

		Map<Long, List<ApiAlbumClassifyVo>> map2 = new HashMap<Long, List<ApiAlbumClassifyVo>>();
		for (AlbumClassify c2 : classifies2) {
			if (map2.containsKey(c2.getParentId())) {
				List<ApiAlbumClassifyVo> apiClassifyVos = map2.get(c2.getParentId());
				ApiAlbumClassifyVo apiClassifyVo = new ApiAlbumClassifyVo();
				apiClassifyVo = copyBean(apiClassifyVo, c2);
				apiClassifyVo.setChildren(map3.get(c2.getId()));
				apiClassifyVos.add(apiClassifyVo);
			} else {
				List<ApiAlbumClassifyVo> apiClassifyVos = new ArrayList<ApiAlbumClassifyVo>();
				ApiAlbumClassifyVo apiClassifyVo = new ApiAlbumClassifyVo();
				apiClassifyVo = copyBean(apiClassifyVo, c2);
				apiClassifyVo.setChildren(map3.get(c2.getId()));
				apiClassifyVos.add(apiClassifyVo);
				map2.put(c2.getParentId(), apiClassifyVos);
			}
		}
		List<ApiAlbumClassifyVo> classifies = page.getRecords();
		int size = classifies.size();
		ApiAlbumClassifyVo c1 = null;

		for (int i = 0; i < size; i++) {
			c1 = classifies.get(i);
			c1.setChildren(map2.get(c1.getId()));
			logger.info("这是 分页 接口 返回的 page 里的 sort值为：=====" + c1.getName() + "=======================" + c1.getSort());
			// 设置上一个分类主键ID
			if (i > 0) {
				c1.setMoveUpId(classifies.get(i - 1).getId());
			}
			// 设置下一个分类主键ID
			if (i < size - 1) {
				c1.setMoveDownId(classifies.get(i + 1).getId());
			}
		}

		// 设置上一个和下一个ID
		Page<ApiAlbumClassifyVo> pageTow = null;
		if (pageNo > 1) {
			pageTow = albumClassifyService.selectListByCompanyId(companyId, name, orgIds, siteId, pageNo - 1, pageSize);
			classifies.get(0).setMoveUpId(pageTow.getRecords().get(pageTow.getRecords().size() - 1).getId());
		}
		pageTow = albumClassifyService.selectListByCompanyId(companyId, name, orgIds, siteId, pageNo + 1, pageSize);
		if (pageTow != null && pageTow.getRecords().size() > 0) {
			classifies.get(classifies.size() - 1).setMoveDownId(pageTow.getRecords().get(0).getId());
		}
		return page;
	}

	private ApiAlbumClassifyVo copyBean(ApiAlbumClassifyVo apiClassifyVo, AlbumClassify classify) {
		apiClassifyVo.setCompanyId(classify.getCompanyId());
		apiClassifyVo.setCreateById(classify.getCreateById());
		apiClassifyVo.setCreateByName(classify.getCreateByName());
		apiClassifyVo.setCreateTime(classify.getCreateTime());
		apiClassifyVo.setDescription(classify.getDescription());
		apiClassifyVo.setId(classify.getId());
		apiClassifyVo.setLayer(classify.getLayer());
		apiClassifyVo.setName(classify.getName());
		apiClassifyVo.setCode(classify.getCode());
		apiClassifyVo.setOrgId(classify.getOrgId());
		apiClassifyVo.setParentId(classify.getParentId());
		apiClassifyVo.setPath(classify.getPath());
		apiClassifyVo.setRootId(classify.getRootId());
		apiClassifyVo.setSiteId(classify.getSiteId());
		apiClassifyVo.setSort(classify.getSort());
		apiClassifyVo.setStatus(classify.getStatus());
		apiClassifyVo.setUpdateById(classify.getUpdateById());
		apiClassifyVo.setUpdateByName(classify.getUpdateByName());
		apiClassifyVo.setUpdateTime(classify.getUpdateTime());
		return apiClassifyVo;
	}

	/**
	 * 验证同站点下 同级的分类不能重名
	 *
	 * @param name      分类名
	 * @param layer     分类层级
	 * @param companyId 企业名
	 * @param siteId    站点id
	 * @param id        分类id （修改的时候传入）
	 * @return Boolean
	 */
	@GetMapping("/vail/name")
	public Boolean vailName(@RequestParam(name = "name") String name, @RequestParam(name = "layer") Integer layer,
			@RequestParam(name = "companyId") Long companyId, @RequestParam(name = "siteId") Long siteId,
			@RequestParam(name = "id", required = false) Long id) {
		AlbumClassify classify = new AlbumClassify();
		classify.setName(name);
		classify.setLayer(layer);
		classify.setCompanyId(companyId);
		classify.setSiteId(siteId);
		EntityWrapper<AlbumClassify> entityWrapper = new EntityWrapper<>(classify);
		if (null != id) {
			entityWrapper.ne("id", id);
		}
		int count = albumClassifyService.selectCount(entityWrapper);
		if (count > 0) {
			return Boolean.FALSE;
		}
		return Boolean.TRUE;
	}

	@GetMapping("/getClassifyName")
	public String getClassifyName(@RequestParam(name = "classifyId") Long classifyId) {
		// 专辑分类回写
		String classifyName = "";
		AlbumClassify classify = albumClassifyService.selectById(classifyId);
		if (classify != null && !StringUtils.isEmpty(classify.getPath())) {

			Set<Long> idsList = new HashSet<>();
			if (classify.getRootId() != 0) {
				idsList.add(classify.getRootId());
			}
			if (classify.getParentId() != 0) {
				idsList.add(classify.getParentId());
			}
			idsList.add(classify.getId());
			List<AlbumClassify> classifies = albumClassifyService.selectClassifyByPath(new ArrayList(idsList));
			for (AlbumClassify classify1 : classifies) {
				if ("".equals(classifyName.toString())) {
					classifyName += classify1.getId();
				} else {
					classifyName += ",";
					classifyName += classify1.getId();
				}
			}
		}
		return classifyName;
	}
	
    @GetMapping("/classify/list")
    public List<AlbumClassifyVo> listClassifyInfo(
            @RequestParam(value="classifyId",required = false) Long classifyId){
        RequestContext rc = ContextHolder.get();
        AlbumClassify classify = new AlbumClassify();
        classify.setCompanyId(rc.getCompanyId());
        classify.setSiteId(rc.getSiteId());
        classify.setStatus(1);
        if(Objects.isNull(classifyId)){
            classify.setLayer(1);
        }else {
            classify.setParentId(classifyId);
        }

        List<AlbumClassify> retData = albumClassifyService.selectList(QueryUtil.condition(classify).
                orderBy("sort",false));
        if(!CollectionUtils.isEmpty(retData)){
            Set<Long> cids = retData.stream().map(item->item.getId()).collect(Collectors.toSet());
            List<AlbumClassifyVo> existVos = albumClassifyService.findExistChildNodes(new ArrayList<>(cids));
            Map<Long,Integer> exitsMap = null;
            boolean isExits = false;
            if(!CollectionUtils.isEmpty(existVos)){
                isExits = true;
                exitsMap = existVos.stream().collect(Collectors.toMap(key->key.getId(),val->val.getTotal()));
            }
            List<AlbumClassifyVo> data = new ArrayList<>();
            for(AlbumClassify item : retData){
            	AlbumClassifyVo vo = new AlbumClassifyVo();
                BeanUtils.copyProperties(item,vo);
                if(isExits && exitsMap.containsKey(vo.getId())){
                    vo.setExistChildNode(true);
                }
                data.add(vo);
            }
            return data;
        }
        return  null;

    }
}
