package com.yizhi.album.application.service;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.IService;
import com.yizhi.album.application.domain.AlbumClassify;
import com.yizhi.album.application.vo.AlbumClassifyVo;
import com.yizhi.album.application.vo.ApiAlbumClassifyVo;
import com.yizhi.core.application.vo.DroolsVo;

import java.util.List;



/**
 * <p>
 * 专辑分类 服务类
 * </p>
 *
 * @author hc123
 * @since 2019-12-30
 */
public interface IAlbumClassifyService extends IService<AlbumClassify> {

    /**
     * 检查是否存在分类名称
     * @param name 分类名称
     * @param parentId 分类的上级id（一级分类为0）
     * @param siteId 站点id
     * @param type 检查类型（1：新增，2：修改(需要传classifyId)）
     * @param classifyId 分类id，修改检查时用到
     * @return 不存在返回true，存放相同名返回false
     */
    boolean checkExtisClassifyName(String name, Long parentId,Long siteId,
                                   int type, Long classifyId);
    /**
     * 添加一个新的课程分类
     * @param classify
     * @return
     */
    int saveClassify(AlbumClassify classify);

    /**
     * 根据父级分类查询是否存在子分类
     * @param classifyIds
     * @return
     */
    List<AlbumClassifyVo> findExistChildNodes(List<Long> classifyIds);


    /**
     * 根据id,删除分类
     * @param id
     * @return
     */
    int deleteClassify(Long id);

    /**
     * 根据分类对象修改分类
     * @param classify
     * @return
     */
    int updateClassify(AlbumClassify classify);


    /**
     * 根据名称查询分类
     * @param name
     * @return
     */
    Page<AlbumClassify> listClassify(String name, Integer pageNo, Integer pageSize,Long companyId,List<Long> orgIds,Long siteId,Boolean isStartHQ );


    /**
     * 学员端查询课程分类
     * @return
     */
    List<AlbumClassify> listStuClassify(Long companyId,Long siteId);

    /**
     * 更换排序
     * @param list
     * @return int
     */
    int updateClassiyfsort(List<AlbumClassify> list);

    /**
     *  管理端专辑分类分页查询
     * @param companyId 企业id
     * @param name      分类名
     * @param orgIds    管辖区 部门id list
     * @param pageNo    分页 第几页
     * @param pageSize  分页数
     * @return Page<ApiClassifyVo>
     */
    Page<ApiAlbumClassifyVo> selectListByCompanyId( Long companyId,String name,List<Long> orgIds,Long siteId,Integer pageNo,Integer pageSize);


    List<AlbumClassify> selectClassifyByPath(List<Long> ids);


    /**
     *  根据分类层级查询分类
     * @param companyId  企业id
     * @param siteId   站点id
     * @param layer    分类层级
     * @return
     */
    List<AlbumClassify> selectClassifyByLayer(Long companyId, Long siteId,Integer layer);

    Page<DroolsVo> getClassifyNameByDrools(String field, String value, Page<DroolsVo> page);
}
