package com.yizhi.album.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.album.application.domain.AlbumClassify;
import com.yizhi.album.application.mapper.AlbumClassifyMapper;
import com.yizhi.album.application.service.IAlbumClassifyService;
import com.yizhi.album.application.vo.AlbumClassifyVo;
import com.yizhi.album.application.vo.ApiAlbumClassifyVo;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.vo.DroolsVo;
import org.apache.ibatis.session.RowBounds;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 专辑分类 服务实现类
 * </p>
 *
 * @author hc123
 * @since 2019-12-30
 */
@Service
public class AlbumClassifyServiceImpl extends ServiceImpl<AlbumClassifyMapper, AlbumClassify> implements IAlbumClassifyService {

    private Logger logger = LoggerFactory.getLogger(AlbumClassifyServiceImpl.class);

    @Autowired
    private AlbumClassifyMapper classifyMapper;

    @Autowired
    private IdGenerator idGenerator;
    
	@Override
	public boolean checkExtisClassifyName(String name, Long parentId, Long siteId, int type, Long classifyId) {
        AlbumClassify classify = new AlbumClassify();
        classify.setName(name);
        classify.setStatus(1);
        classify.setParentId(parentId == null ? 0 : parentId.longValue());
        classify.setSiteId(siteId);
        EntityWrapper wrapper = new EntityWrapper(classify);
        if (type == 2) {
            wrapper.ne("id", classifyId);
        }
        return classifyMapper.selectCount(wrapper) == 0;
	}

	@Override
	public int saveClassify(AlbumClassify classify) {
		AlbumClassify exitsNameClassify = new AlbumClassify();
        exitsNameClassify.setName(classify.getName());
        exitsNameClassify.setParentId(classify.getParentId());
        exitsNameClassify.setStatus(1);
        exitsNameClassify.setSiteId(classify.getSiteId());
        int count = exitsNameClassify.selectCount(new EntityWrapper(exitsNameClassify));
        // 如果同级下存在相同名称的分类，返回标识符3
        if (count > 0) {
            return 3;
        }
        Long id = idGenerator.generate();
        classify.setId(id);
        Integer sort = classifyMapper.selectSort(classify.getParentId());
        classify.setSort(++sort);
        if (classify.getParentId() == 0L) {
            classify.setLayer(1);
            classify.setRootId(id);
            classify.setPath("" + id);
        } else {
            int layer = classifyMapper.selectParentLayer(classify.getParentId());
            if (layer > 2) {
                logger.error("添加超过三级");
                return 0;
            }
            if (layer == 1) {
                classify.setPath(classify.getParentId() + "," + id);
                classify.setLayer(++layer);
                classify.setRootId(classify.getParentId());
            } else {
                classify.setLayer(++layer);
                Long rootid = classifyMapper.selectRootid(classify.getParentId());
                classify.setRootId(rootid);
                classify.setPath(rootid + "," + classify.getParentId() + "," + id);
            }
        }
        return classifyMapper.insert(classify);
	}

	@Override
	public List<AlbumClassifyVo> findExistChildNodes(List<Long> classifyIds) {
        List<AlbumClassifyVo> data = null;
        data = this.baseMapper.findExistChildNodes(classifyIds);
        return data;
	}

	@Override
	public int deleteClassify(Long id) {
		AlbumClassify classify = new AlbumClassify();
        classify.setId(id);
        //查找有没有子级分类
        Long childid = classifyMapper.selectChild(id);
        int i = 0;
        if (childid == null) {
            //查找有没有挂有专辑
            Long i1 = classifyMapper.selectAlbum(id, ContextHolder.get().getSiteId());
            if (i1 == null) {
                try {
                    classify.setStatus(0);
                    i = classifyMapper.updateById(classify);
                } catch (Exception e) {
                    logger.error("删除失败", e);
                    return 0;
                }
            }
            else{ return -1;} //分类下挂有专辑需要在web进行提示
        }
        return i;
	}

	@Override
	public int updateClassify(AlbumClassify classify) {
        try {
        	AlbumClassify oldClassify = classify.selectById();
            if (oldClassify == null) {
                return 0;
            }
            AlbumClassify existUpdateClassify = new AlbumClassify();
            existUpdateClassify.setParentId(oldClassify.getParentId());
            existUpdateClassify.setName(classify.getName());
            existUpdateClassify.setSiteId(classify.getSiteId());
            EntityWrapper<AlbumClassify> ew = new EntityWrapper(existUpdateClassify);
            ew.notIn("id", oldClassify.getId());
            int count = existUpdateClassify.selectCount(ew);
            // 如果同级下存在相同名称的分类，返回标识符3
            if (count > 0) {
                return 3;
            }
            return classifyMapper.updateById(classify);
        } catch (Exception e) {
            logger.error("修改分类失败");
            return 0;
        }
	}

	@Override
	public Page<AlbumClassify> listClassify(String name, Integer pageNo, Integer pageSize, Long companyId,
			List<Long> orgIds, Long siteId, Boolean isStartHQ) {
        List<String> list = new ArrayList<String>();
        list.add("path");
        Page<AlbumClassify> page = new Page<AlbumClassify>(pageNo, pageSize);
        AlbumClassify classify = new AlbumClassify();
        classify.setStatus(1);
        classify.setCompanyId(companyId);
        classify.setSiteId(siteId);
        EntityWrapper<AlbumClassify> entityWrapper = new EntityWrapper<AlbumClassify>(classify);
        entityWrapper.orderAsc(list);
        if (StringUtils.isEmpty(name)) {
            entityWrapper.like("name", name);
        }
        // 分级授权修改
        if(isStartHQ) HQueryUtil.startHQ(AlbumClassify.class);
        page = this.selectPage(page, entityWrapper);
        return page;
	}

	@Override
	public List<AlbumClassify> listStuClassify(Long companyId, Long siteId) {
        try {
            List<String> list = new ArrayList<String>();
            list.add("path");
            AlbumClassify classify = new AlbumClassify();
            classify.setStatus(1);
            classify.setCompanyId(companyId);
            classify.setSiteId(siteId);
            EntityWrapper<AlbumClassify> entityWrapper = new EntityWrapper<AlbumClassify>(classify);
            entityWrapper.orderAsc(list);
            List<AlbumClassify> classifies1 = classifyMapper.selectList(entityWrapper);
            return classifies1;
        } catch (Exception e) {
            logger.error("", e);
            return null;
        }
	}

	@Override
	public int updateClassiyfsort(List<AlbumClassify> list) {
        if (list.size() == 2) {
        	AlbumClassify classifyFirst = list.get(0);
        	AlbumClassify classifyLast = list.get(1);
            if(classifyFirst.getSort().equals(classifyLast.getSort())){
                classifyLast.setSort(classifyLast.getSort().intValue()-1);
            }
            classifyMapper.updateById(classifyFirst);
            classifyMapper.updateById(classifyLast);
        }
        return 1;
	}

	@Override
	public Page<ApiAlbumClassifyVo> selectListByCompanyId(Long companyId, String name, List<Long> orgIds, Long siteId,
                                                          Integer pageNo, Integer pageSize) {
        Page<ApiAlbumClassifyVo> page = new Page<ApiAlbumClassifyVo>(pageNo, pageSize);
        // 分级授权修改
        HQueryUtil.startHQ(AlbumClassifyServiceImpl.class);
        List<ApiAlbumClassifyVo> apiClassifyVos = this.baseMapper.selectListByCompanyId(companyId, name, orgIds,siteId, new RowBounds(page.getOffset(), page.getLimit()));
        Integer count = this.baseMapper.selectCountByCompanyId(companyId, name, orgIds,siteId);
        page.setTotal(count);
        page.setRecords(apiClassifyVos);
        return page;
	}

	@Override
	public List<AlbumClassify> selectClassifyByPath(List<Long> ids) {
        return this.baseMapper.selectClassifyByRootId(ids);
	}

	@Override
	public List<AlbumClassify> selectClassifyByLayer(Long companyId, Long siteId, Integer layer) {
        // 分级授权修改
        HQueryUtil.startHQ(AlbumClassifyServiceImpl.class);
        return this.baseMapper.selectClassifyByLayer(companyId,siteId,layer);
	}

    @Override
    public Page<DroolsVo> getClassifyNameByDrools(String field, String value, Page<DroolsVo> page) {
        if (StringUtils.isEmpty(field)) {
            logger.info("列名不能为空！");
            return page;
        }
        RequestContext requestContext = ContextHolder.get();
        Long siteId = requestContext.getSiteId();
        Long companyId = requestContext.getCompanyId();
        List<DroolsVo> voList = null;

        AlbumClassify classify = new AlbumClassify();
        classify.setSiteId(siteId);
        classify.setCompanyId(companyId);
        classify.setStatus(1);//0:删除 1：上架
        EntityWrapper wrapper = new EntityWrapper(classify);
        wrapper.setSqlSelect("distinct(" + field + "),"+"id ")
                .isNotNull(field)
                .like(field, value)
                .addFilter(field+"!=''")
                .orderBy("create_time", false);

        List<AlbumClassify> list = this.baseMapper.selectPage(page, wrapper);
        if (!CollectionUtils.isEmpty(list)){
            voList = new ArrayList<>(list.size());
            for (AlbumClassify a:list){
                DroolsVo vo = new DroolsVo();
                vo.setTaskId(a.getId());
                vo.setTaskFieldValue(a.getName());
                vo.setTaskParamsType(field);
                voList.add(vo);
            }
        }
        page.setRecords(voList);
        return page;
    }
}
