package com.yizhi.album.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.album.application.domain.AlThemeActivity;
import com.yizhi.album.application.domain.Album;
import com.yizhi.album.application.mapper.AlStudentActivityRecordMapper;
import com.yizhi.album.application.mapper.AlThemeActivityMapper;
import com.yizhi.album.application.mapper.AlbumMapper;
import com.yizhi.album.application.mapper.AlbumVisibleRangeMapper;
import com.yizhi.album.application.service.IAlbumClassifyService;
import com.yizhi.album.application.service.IAlbumService;
import com.yizhi.album.application.vo.*;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.enums.TaskParamsEnums;
import com.yizhi.core.application.vo.DroolsVo;
import com.yizhi.course.application.feign.CoursePcClient;
import com.yizhi.util.application.clazz.ClassUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 专辑表 服务实现类
 * </p>
 *
 * @author hc123
 * @since 2019-12-30
 */
@Service
public class AlbumServiceImpl extends ServiceImpl<AlbumMapper, Album> implements IAlbumService {

    private static final Logger LOGGER = LoggerFactory.getLogger(AlbumServiceImpl.class);

	@Autowired
	IdGenerator idGenerator;

	@Autowired
	private AlbumVisibleRangeMapper albumVisibleRangeMapper;

	@Autowired
	private AlStudentActivityRecordMapper alStudentActivityRecordMapper;

    @Autowired
    private CoursePcClient coursePcClient;

    @Autowired
    private AlThemeActivityMapper alThemeActivityMapper;
    @Autowired
	private IAlbumClassifyService albumClassifyService;
    @Autowired
	private AlbumMapper albumMapper;

	@Override
	public Page<Album> searchPage(String name, Long alClassifyId, Integer status, Integer overed, Long companyId,
			Long siteId, Integer pageNo, Integer pageSize) {
        Page<Album> page = new Page<Album>(pageNo, pageSize);
        // 分级授权修改
        HQueryUtil.startHQ(Album.class);
        List<Album> list = this.baseMapper.searchPage(name, alClassifyId, status, overed, companyId, siteId, page);
        page.setRecords(list);
		return page;
	}

	@Override
	public AlbumVo save(AlbumVo albumVo) {
		if (!this.vaidName(albumVo.getId(), albumVo.getName(), albumVo.getSiteId())) {
            return null;
		}
		try {
			Album album = null;
			if (albumVo.getId()!=null) {
				album = this.baseMapper.selectById(albumVo.getId());
			}else {
				album = new Album();
				album.setId(idGenerator.generate());
				album.setCompanyId(albumVo.getCompanyId());
				album.setSiteId(albumVo.getSiteId());
				album.setOrgId(albumVo.getOrgId());
				album.setCreateById(albumVo.getCreateById());
				album.setCreateByName(albumVo.getCreateByName());
				album.setCreateTime(albumVo.getCreateTime());
				//专辑规则初始化默认值
				album.setShelves(2);
				album.setScope(1);
				album.setCommentEnable(0);
				album.setDocumentEnable(0);
				album.setDelFlg(0);
				album.setOvered(0);
				album.setDisplay(0);
				album.setUpdateById(albumVo.getCreateById());
				album.setUpdateByName(albumVo.getCreateByName());
				album.setUpdateTime(albumVo.getCreateTime());
			}
			album.setName(albumVo.getName());
			album.setClassifyId(albumVo.getClassifyId());
			album.setImage(null==albumVo.getImage()||""==albumVo.getImage()?"http://customproject.oss-cn-shanghai.aliyuncs.com/yzphoto/album_defalut_logo.png":albumVo.getImage());
			album.setDescription(albumVo.getDescription());
			album.setTags(albumVo.getTags());
			this.insertOrUpdate(album);
			albumVo.setId(album.getId());
			albumVo.setOvered(album.getOvered());
			return albumVo;
		} catch (Exception e) {
            LOGGER.info("新增或修改专辑异常" + JSON.toJSONString(e));
            System.out.println(e.getLocalizedMessage());
            return null;
		}
	}

	@Override
	public Boolean vaidName(Long id, String name, Long siteId) {
		Integer count = this.baseMapper.vailName(id, name, siteId);
		if (count>0) {
			return Boolean.FALSE;
		}
		return Boolean.TRUE;
	}

	@Override
	public Boolean update(AlbumVo albumVo) {
		try {
			Album album = this.baseMapper.selectById(albumVo.getId());
			album.setScope(albumVo.getScope());
			album.setDocumentEnable(albumVo.getDocumentEnable());
			album.setCommentEnable(albumVo.getCommentEnable());
			album.setOvered(albumVo.getOvered());
			album.setDisplay(albumVo.getDisplay());
			album.setEnableTask(albumVo.getEnableTask());
			album.setTaskStartDate(albumVo.getTaskStartDate());
			album.setTaskEndDate(albumVo.getTaskEndDate());
			album.setUpdateById(albumVo.getUpdateById());
			album.setUpdateByName(albumVo.getUpdateByName());
			album.setUpdateTime(albumVo.getUpdateTime());
			this.baseMapper.updateById(album);
		} catch (Exception e) {
            LOGGER.info("更新错误消息={}",JSON.toJSONString(e));
            return Boolean.FALSE;
		}
		return Boolean.TRUE;
	}

	@Override
	public List<Album> searchlList(String name,List<Long> ids, Long companyId, Long siteId) {
		// TODO Auto-generated method stub
		return this.baseMapper.selectAlbumList(name,ids, companyId, siteId);
	}

	@Override
	public Page<ApiListAlbumVo> searchPageSubClassify(String name, List<Long> classifyIds, List<Long> relationIds,
													  Long companyId, Long siteId, int pageNo, int pageSize) {
        List<Long> ids = null;
        if (relationIds != null && relationIds.size() > 0) {
            ids = albumVisibleRangeMapper.selectAlbumIdByRelationId(relationIds);
        }
        Page<ApiListAlbumVo> page = new Page<ApiListAlbumVo>(pageNo, pageSize);
        List<ApiListAlbumVo> data = this.baseMapper.searchPageSubClassify(name, classifyIds, ids,
                companyId, siteId, page);
        if (CollectionUtils.isNotEmpty(data)) {
        	for(ApiListAlbumVo vo:data) {
				Integer overed = vo.getOvered();
				if (null != overed && overed.equals(1)){
					//获取已完结状态下的所有课程数量
					vo.setCourseNum(0);
					Long totalCourseCount = baseMapper.getAlbumTotalCourseCount(vo.getId());
					if (null != totalCourseCount){
						vo.setCourseNum(totalCourseCount.intValue());
					}
				}
				List<Long> courseIds = this.baseMapper.getCourseIds(vo.getId());
        		if (CollectionUtils.isNotEmpty(courseIds)) {
            		Map<Long, Long> durations = coursePcClient.getMaterialTimeByCourseIds(courseIds);
            		Long duration = (long) 0;
                    if (null != durations && durations.size() > 0) {
                        for (Long h : durations.keySet()) {
                            LOGGER.info("课程时长：" + durations.get(h));
                            duration = duration + (durations.get(h) != null ? durations.get(h) : 0l);
                        }
                    }
                    vo.setDuration(duration);
				}else {
					vo.setDuration(0L);
				}
        	}
            List<Long> albumIds = data.stream().map(ApiListAlbumVo::getId).collect(Collectors.toList());
            List<ApiListAlbumVo> vos = alStudentActivityRecordMapper.getStudentNumByIds(albumIds);
            LOGGER.info("专辑学习人数={}", JSON.toJSONString(vos));
            if (CollectionUtils.isNotEmpty(vos)) {
                Map<Long, Integer> dataMap = vos.stream().collect(Collectors.
                        toMap(key -> key.getId(), val -> val.getStudyNum()));
                for (ApiListAlbumVo itemAlbum : data) {
                	itemAlbum.setStudyNum(dataMap.get(itemAlbum.getId()));
                }
            }
        }
        page.setRecords(data);
        return page;
	}

	@Override
	public List<Long> getMyStudyList(Long siteId, Long accountId,Page<Long> page) {
		// TODO Auto-generated method stub
		return this.baseMapper.getMyStudyList(siteId, accountId,page);
	}

	@Override
	public Integer finished(List<Long> ids,Long albumId, Long accountId, Long siteId) {
		// TODO Auto-generated method stub
		return this.baseMapper.finished(ids,albumId, accountId, siteId);
	}

	@Override
	public Integer unLockNum(Long albumId) {
		// TODO Auto-generated method stub
		return this.baseMapper.unLockNum(albumId);
	}

	@Override
	public Integer getStudyNum(Long albumId) {
		// TODO Auto-generated method stub
		return alStudentActivityRecordMapper.getStudyNum(albumId);
	}

	@Override
	public List<AlbumThemeVo> getThemes(Long albumId, Integer type) {
		// TODO Auto-generated method stub
		return this.baseMapper.geThemeVos(albumId, type);
	}

	@Override
	public List<AlbumActivityVo> getActivitys(Long themeId, Integer type, Integer display) {
		// TODO Auto-generated method stub
		return this.baseMapper.getActivityVos(themeId, type, display);
	}

	@Override
	public void unLockCourse() {
		Date date = new Date();
		List<Long> ids = this.baseMapper.getLockList();
		if (CollectionUtils.isNotEmpty(ids)) {
			for(Long id : ids) {
				AlThemeActivity activity = alThemeActivityMapper.selectById(id);
				if (activity.getUnlockTime().before(date)) {
					activity.setUnlock(0);
					alThemeActivityMapper.updateById(activity);
					Album album = this.baseMapper.selectById(activity.getAlbumId());
					album.setUpdateTime(date);
					this.baseMapper.updateById(album);
				}
			}
		}
	}

	@Override
	public Page<ApiListAlbumVo> getMySubscribePage(List<Long> ids, int pageNo, int pageSize) {
        Page<ApiListAlbumVo> page = new Page<ApiListAlbumVo>(pageNo, pageSize);
		List<ApiListAlbumVo> data = this.baseMapper.getMySubscribePage(ids,page);
		for(ApiListAlbumVo vo:data) {
			//获取完结状态的课程数
			Integer overed = vo.getOvered();
			if (null != overed && overed.equals(1)){
				vo.setCourseNum(0);
				Long albumTotalCourseCount = baseMapper.getAlbumTotalCourseCount(vo.getId());
				if (null != albumTotalCourseCount){
					vo.setCourseNum(albumTotalCourseCount.intValue());
				}
			}
			if (vo.getUnLockTime()!=null) {
				long day = ((System.currentTimeMillis()-vo.getUnLockTime().getTime())/(3600*24*1000));
				vo.setIsNew(day>5?0:1);
			}else {
				vo.setIsNew(0);
			}
		}
		page.setRecords(data);
		return page;
	}

	@Override
	public Page<DroolsVo> getPageByDrools(String field, String value, Page<DroolsVo> page) {
		if (StringUtils.isBlank(field)) {
			LOGGER.info("列名不能为空！");
			return page;
		}
		if (field.equalsIgnoreCase(TaskParamsEnums.CLASSIFY.getCode())) {
			field = "name";
			page = albumClassifyService.getClassifyNameByDrools(field, value, page);
		} else {
			if (field.equalsIgnoreCase(TaskParamsEnums.NAME.getCode())) {
				return getPage(field, value, page);
			} else if (field.equalsIgnoreCase(TaskParamsEnums.KEYWORD.getCode())) {
				field = "tags";
				return getPage(field, value, page);
			}
		}
		return page;
	}

	public Page getPage(String field, String value, Page<DroolsVo> page) {
		RequestContext requestContext = ContextHolder.get();
		Long siteId = requestContext.getSiteId();
		Long companyId = requestContext.getCompanyId();

		Album album = new Album();
		album.setSiteId(siteId);
		album.setCompanyId(companyId);
		album.setDelFlg(0);
		album.setShelves(1);
		EntityWrapper wrapper = new EntityWrapper(album);
		wrapper.setSqlSelect("distinct(" + field + "),"+"id ")
				.isNotNull(field)
				.like(field, value)
				.addFilter(field+"!=''")
				.orderBy("create_time", false);

		String upperField = ClassUtil.getFieldName(field);
		List<DroolsVo> voList = null;
		List<Album> list = this.baseMapper.selectPage(page, wrapper);
		if (!CollectionUtils.isEmpty(list)) {
			voList = new ArrayList<>(list.size());
			for (Album a : list) {
				DroolsVo vo = new DroolsVo();
				vo.setTaskId(a.getId());
				vo.setTaskFieldValue(ClassUtil.invokeMethod(a, upperField));
				vo.setTaskParamsType(field);
				voList.add(vo);
			}
			page.setRecords(voList);
		}
		return page;
	}

	@Override
	public Page<AlbumCalendarVo> getPageToCalendar(Date date, Page<Album> page) {
		RequestContext context = ContextHolder.get();
		Long siteId = context.getSiteId();
		Long accountId = context.getAccountId();
		Page<AlbumCalendarVo> voPage = new Page<>();
		BeanUtils.copyProperties(page,voPage);

		//获取该时间参数的专辑ds
        List<Long> ids = albumMapper.getIdsByDate(date, siteId);
        if (CollectionUtils.isEmpty(ids)) {
            return voPage;
        }
        List<Album> albums = this.selectBatchIds(ids);
        //获取已完结的达到100%的专辑ids
        Set<Long> hasFinishedIds = new HashSet(ids.size());
        albums.forEach(a -> {
            //只有已完结的专辑需要计算
            if (a.getOvered().equals(1)) {
                // 计算完成百分比
                List<Long> activityIds = albumMapper.getIdsByAlbumId(a.getId());
                Integer finished = this.finished(activityIds, a.getId(), accountId, siteId);
                Integer unLockNum = this.unLockNum(a.getId());
                if (finished != null && unLockNum != null && unLockNum != 0) {
                    if (finished.equals(unLockNum)) {
                        hasFinishedIds.add(a.getId());
					}
				}
			}
		});
		//去除已完结且达到100%的id
		ids.removeAll(hasFinishedIds);
		//根据可见范围获取专辑ids
		List<Long> albumIdsFromVisibeRange = albumVisibleRangeMapper.getUsefulIds(ids, context.getRelationIds(), siteId);


		//获取最终数据
		List<Album> pageToCalendar = albumMapper.getPageToCalendar(new ArrayList<>(hasFinishedIds), albumIdsFromVisibeRange, date, siteId, page);
		voPage.setTotal(albumMapper.getPageToCalendarNum(new ArrayList<>(hasFinishedIds),albumIdsFromVisibeRange,date,siteId));
		voPage.setRecords(buildAlbumCalendarVo(pageToCalendar));
		return voPage;
	}

	public List<AlbumCalendarVo> buildAlbumCalendarVo(List<Album> pageToCalendar) {
		List<AlbumCalendarVo> albumCalendarVos = null;
		if (CollectionUtils.isNotEmpty(pageToCalendar)) {
			albumCalendarVos = new ArrayList<>(pageToCalendar.size());
			for (Album a : pageToCalendar) {
				AlbumCalendarVo vo = new AlbumCalendarVo();
				BeanUtils.copyProperties(a,vo);
				albumCalendarVos.add(vo);
			}
		}
		return albumCalendarVos;
	}
}
